"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Set = void 0;
const base_collection_1 = require("./base-collection");
const hash_utils_1 = require("./hash-utils");
class Node {
    value;
    next;
    constructor(value) {
        this.value = value;
        this.next = null;
    }
}
class Set extends base_collection_1.BaseCollection {
    table;
    count;
    capacity;
    constructor(capacity = 81920) {
        super();
        this.capacity = capacity <= 0 ? 81920 : capacity;
        this.table = new Array(this.capacity).fill(null);
        this.count = 0;
    }
    /**
     * Adds a value to the set if it's not already present.
     *
     * TODO: Dinamically resize the table if the collision factor is too high.
     */
    insert(value) {
        const index = hash_utils_1.HashUtils.hash(value, this.capacity);
        // Handle empty bucket case.
        if (!this.table[index]) {
            this.table[index] = new Node(value);
            this.count++;
            return;
        }
        // Check first node for value match. If it matches, do nothing.
        if (hash_utils_1.HashUtils.equals(this.table[index].value, value))
            return;
        // Traverse chain to find value or last node. If it matches, do nothing.
        let current = this.table[index];
        while (current?.next) {
            if (hash_utils_1.HashUtils.equals(current.next.value, value))
                return;
            current = current.next;
        }
        // Value not found, append new node.
        current.next = new Node(value);
        this.count++;
    }
    /**
     * Adds multiple values to the set if they're not already present.
     */
    insertList(values) {
        for (const value of values) {
            this.insert(value);
        }
    }
    /**
     * Checks if a value exists in the set.
     */
    find(value) {
        const index = hash_utils_1.HashUtils.hash(value, this.capacity);
        let current = this.table[index];
        while (current) {
            if (hash_utils_1.HashUtils.equals(current.value, value))
                return true;
            current = current.next;
        }
        return false;
    }
    has(value) {
        return this.find(value);
    }
    /**
     * Removes a value from the set.
     */
    remove(value) {
        const index = hash_utils_1.HashUtils.hash(value, this.capacity);
        // Handle empty bucket case
        if (!this.table[index])
            return false;
        // Handle first node case
        if (hash_utils_1.HashUtils.equals(this.table[index].value, value)) {
            this.table[index] = this.table[index].next;
            this.count--;
            return true;
        }
        // Handle remaining nodes
        let current = this.table[index].next;
        let prev = this.table[index];
        while (current) {
            if (hash_utils_1.HashUtils.equals(current.value, value)) {
                prev.next = current.next;
                this.count--;
                return true;
            }
            prev = current;
            current = current.next;
        }
        return false;
    }
    forEach(callback) {
        for (const node of this.table) {
            let current = node;
            while (current) {
                callback(current.value);
                current = current.next;
            }
        }
    }
    /**
     * Removes all elements from the set.
     */
    clear() {
        this.table = new Array(this.capacity).fill(null);
        this.count = 0;
    }
    /**
     * Returns true if the set contains no elements.
     */
    isEmpty() {
        return this.count === 0;
    }
    /**
     * Returns the number of elements in the set.
     */
    size() {
        return this.count;
    }
}
exports.Set = Set;
//# sourceMappingURL=set.js.map